<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

trait Functions {

	public function canInitDepoClass() {

		if(
			isset(
				$this->settings['username'],
				$this->settings['password'],
				$this->settings['size'],
				$this->settings['enviroment'],
				$this->settings['client'],
				$this->settings['debug']
				) and
			!empty($this->settings['client']) and
			!empty($this->settings['username']) and 
			!empty($this->settings['password'])
			) return true;

			return false;
	}


	public function strip_instance( $instance ) {
		$instance = explode(':', $instance);
		return array_shift($instance);
	}


	public function get_trans_name() {
		$session = new WC_Session_Handler;
		$user_id = $session->get_customer_id();
		return OKTO_DEPO_PREFIX . $user_id;		
	}

	public function get_order_hash() {

		$trans_name = $this->get_trans_name();

		$value = false;

		if ( false === ( $value = get_transient( $trans_name ) ) ) {
					    // this code runs when there is no valid transient set
			$value = date('dmY') . '_' . $this->generate_random_string();
			set_transient( $trans_name , $value, HOUR_IN_SECONDS * 6 );
		}

		return $value;
	}

	public function remove_order_hash() {
		delete_transient( $this->get_trans_name() );
	}

	public function toAscii($text) {
		setlocale(LC_ALL, 'en_US.UTF8');
		$clean = iconv('UTF-8', 'ASCII//TRANSLIT', $text);
		$clean = preg_replace("/[^a-zA-Z0-9\/_| -]/", '', $clean);
		$clean = strtolower(trim($clean, '-'));
		$clean = preg_replace("/[\/_| -]+/", '-', $clean);
		return $clean;
	}

	public function generate_random_string($length = 10, $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ') {
		$charactersLength = strlen($characters);
		$randomString = '';
		for ($i = 0; $i < $length; $i++) {
			$randomString .= $characters[rand(0, $charactersLength - 1)];
		}
		return $randomString;
	}

	public function get_payment_gateways() {
		$session = new WC_Session_Handler;
		$gateways = unserialize( $session->get( 'depo_available_payments' ) );
		return $gateways;
	}

	public function set_payment_gateways() {
	        $gateway = new WC_Payment_Gateways();
			$payments = [];
			$i = 0;
			foreach($gateway->payment_gateways() as $pg) {
				$payments[$i]['id'] = $pg->id;
				$payments[$i]['title'] = $pg->title;

				if (is_array($this->settings['cod']) && in_array($pg->id, $this->settings['cod'])) {
                    $payments[$i]['selected'] = 1;
                }
                else {
                    $payments[$i]['selected'] = 0;
                }

				$i++;
			}
//			$session = new WC_Session_Handler;
//			$session->set( 'depo_available_payments', serialize($payments) );
			return $payments;
	}

	public function validate_selected_pickup( $pickup = null ) {
		$pickup = isset($_POST['okto-depo-pickup']) && $_POST['okto-depo-pickup'] != '' ? $this->toAscii($_POST['okto-depo-pickup']) : false;
		if(!$pickup) return false;

		$places = $this->api->get_places_from_file();
		foreach($places as $place) {
			if($this->toAscii($place->id) == $pickup) {
				return $place;
			}
		}

		return false;
	}

	public function arrayToString( $arr = [] ) {
		$return = [];

	    foreach ($arr as $key => $value) {
	        $return[] = stripslashes($key) . "=" . stripslashes($value);
	    }
	    return join("&", $return);
	} 

	public function debug($data = null, $preppend = null, $append = null) {
		if(!$this->debug) return;

		if(is_array($data))
			$data = print_r($data, true);

		file_put_contents( OKTO_DEPO_PATH . OKTO_DEPO_DEBUG, $preppend . $data . $append, FILE_APPEND);
	}

	public function makePhoneGreatAgain( $phone ) {
		$phone = preg_replace("/[^0-9]/", "", $phone);

		if(substr($phone, 0, 1) == '0') 
			$phone = substr($phone, 1);
		 		
		if(substr($phone, 0, 1) == '9') 
			$phone = '421' . $phone;

		return $phone;
	}

	public function print_label( $depo_order_id, $name = '') {
		if($depo_order_id && !isset($depo_order_id->detail)) {

			$name = ($name == '') ? date('d-m-Y-H-i-s') : $name ;

			header('Pragma: public');
		    header('Expires: Mon, 26 Jul 1997 05:00:00 GMT');
		    // header("Content-Disposition: attachment; filename=\"depo_test.pdf\";" );
		    header("Content-Disposition: attachment; filename=\"depo_".$name.".pdf\";" );
			header('Content-Type: application/octet-stream');
			header('Content-Length: ' . strlen($depo_order_id));
			header('Cache-Control: must-revalidate, post-check=0, pre-check=0');

		    echo $depo_order_id;
			return;
		}
	}
}