<?php
		if ( ! class_exists( 'WC_Depo_Shipping_Method' ) ) {

			require_once(  OKTO_DEPO_INC_PATH . 'depo-api.php' );
			require_once(  OKTO_DEPO_INC_PATH . 'functions.php' );

			class WC_Depo_Shipping_Method extends WC_Shipping_Method {

				use Functions;

				public $config;

				public function __construct( $instance_id = 0 ) {
					$this->instance_id        = absint( $instance_id );

					$this->config             = include( OKTO_DEPO_INC_PATH . 'config.php' );

					$this->id                 = $this->config['id']; //'depo_shipping_method';
					
					// $this->id                 = 'depo_shipping_method'; // Id for your shipping method. Should be uunique.
					$this->method_title       = $this->config['method_title']; // __( 'Depo.sk' );  // Title shown in admin
					$this->method_description = $this->config['method_description']; // __( 'Prepavná metóda pre depo.sk' ); // Description shown in admin

					$this->supports           = $this->config['supports']; //array(
					$this->enabled            = "yes"; // This can be added as an setting but for this example its forced enabled
					$this->title              = $this->method_title; //"Depo.sk"; // This can be added as an setting but for this example its forced.

					$this->init();

					add_action( 'woocommerce_update_options_shipping_' . $this->id, array( $this, 'process_admin_options' ) );
				}

				public function init() {

					$this->init_form_fields(); // This is part of the settings API. Override the method to add your own settings

					$this->init_settings(); // This is part of the settings API. Loads settings you previously init.

					if($title = $this->get_instance_option( 'title' ))
						$this->title            = $title;

					$this->tax_status     = $this->get_instance_option( 'tax_status' );
					$this->type           = $this->get_option( 'type', 'class' );

					$this->api = $this->canInitDepoClass() ? new Depo( $this->settings ) : false;
				}

				public function init_form_fields() {

					$available_gateways = $this->get_payment_gateways(); 

					$this->form_fields          = include( OKTO_DEPO_INC_PATH . 'settings.php' );
					$this->instance_form_fields = include( OKTO_DEPO_INC_PATH . 'instance_settings.php' );
				}

				public function calculate_shipping( $package = array() ) {

					$cost = $this->get_instance_option('cost'); // default - non class price
					$classes = [];

					foreach($package['contents'] as $contents) {

						if(!$class_id = $contents['data']->get_shipping_class_id()) 
							continue;

						if( $temp = $this->get_option('class_cost_' . $class_id) ) 
							$classes[$class_id] = $temp; 
					}

					// ak sa pouzivaju klassy
					if(sizeof($classes)) {

						// class - spocitat vsetky postovne dokopy
						// order - najdrahsi class - default
						$cost = ( $this->get_option('calctype') == 'class' ) ? array_sum($classes) : max($classes);
					} 

					$rate = array(
						'id'      => $this->get_rate_id(),
						'label'   => $this->title,
						'cost'    => $cost,
						'taxes'   => true,
						'package' => $package
					);

					// Register the rate
					$this->add_rate( $rate );

					do_action( 'woocommerce_' . $this->id . '_shipping_add_rate', $this, $rate );
				}
			}
		}

        if ( ! class_exists( 'WC_Depo_Shipping_ToAddress_Method' ) ) {

            require_once(OKTO_DEPO_INC_PATH . 'depo-api.php');
            require_once(OKTO_DEPO_INC_PATH . 'functions.php');

            class WC_Depo_Shipping_ToAddress_Method extends WC_Shipping_Method
            {
//                TODO rewrite
                use Functions;

                public $config;

                public function __construct( $instance_id = 0 ) {
                    $this->instance_id        = absint( $instance_id );

                    $this->config             = include( OKTO_DEPO_INC_PATH . 'config.php' );

                    $this->id                 = $this->config['id_shipping_toAddress']; //'depo_shipping_toAddress_method';

                    $this->method_title       = $this->config['method_shippingToAddress_title']; // Title shown in admin
                    $this->method_description = $this->config['method_shippingToAddress_description']; // // Description shown in admin

                    $this->supports           = $this->config['supports']; //array(
                    $this->enabled            = "yes"; // This can be added as an setting but for this example its forced enabled
                    $this->title              = $this->method_title; // This can be added as an setting but for this example its forced.

                    $this->init();

                    add_action( 'woocommerce_update_options_shipping_' . $this->id, array( $this, 'process_admin_options' ) );
                }

                public function init() {

                    $this->init_form_fields(); // This is part of the settings API. Override the method to add your own settings

                    $this->init_settings(); // This is part of the settings API. Loads settings you previously init.

                    if($title = $this->get_instance_option( 'title' ))
                        $this->title            = $title;

                    $this->tax_status     = $this->get_instance_option( 'tax_status' );
                    $this->type           = $this->get_option( 'type', 'class' );

                    $this->api = $this->canInitDepoClass() ? new Depo( $this->settings ) : false;
                }

                public function init_form_fields() {

                    $available_gateways = $this->get_payment_gateways();

                    $this->form_fields          = include( OKTO_DEPO_INC_PATH . 'settings.php' );
                    $this->instance_form_fields = include( OKTO_DEPO_INC_PATH . 'instance_settings.php' );
                }

                public function calculate_shipping( $package = array() ) {

                    $cost = $this->get_instance_option('cost'); // default - non class price
                    $classes = [];

                    foreach($package['contents'] as $contents) {

                        if(!$class_id = $contents['data']->get_shipping_class_id())
                            continue;

                        if( $temp = $this->get_option('class_cost_' . $class_id) )
                            $classes[$class_id] = $temp;
                    }

                    // ak sa pouzivaju klassy
                    if(sizeof($classes)) {

                        // class - spocitat vsetky postovne dokopy
                        // order - najdrahsi class - default
                        $cost = ( $this->get_option('calctype') == 'class' ) ? array_sum($classes) : max($classes);
                    }

                    $rate = array(
                        'id'      => $this->get_rate_id(),
                        'label'   => $this->title,
                        'cost'    => $cost,
                        'taxes'   => true,
                        'package' => $package
                    );

                    // Register the rate
                    $this->add_rate( $rate );

                    do_action( 'woocommerce_' . $this->id . '_shipping_add_rate', $this, $rate );
                }
            }
        }

